from typing import Any, List, Dict, Text


_INITIALS = ...  # type: List[Text]

_INITIALS_NOT_STRICT = ...  # type: List[Text]

_FINALS = ...  # type: List[Text]


PHONETIC_SYMBOL_DICT = ...  # type: Dict[Text, Text]
PHONETIC_SYMBOL_DICT_KEY_LENGTH_NOT_ONE = ...  # type: Dict[Text, Text]

RE_PHONETIC_SYMBOL = ...  # type : Any


RE_TONE2 = ...  # type : Any

RE_TONE3 = ...  # type : Any


RE_NUMBER = ...  # type: Any



from typing import Any, List, Dict, Text

_convert_table = ...  # type: List[List[Text]]

_initial_table = ...  # type: List[List[Text]]

_tone_table = ...  # type: List[List[Text]]

_except_table = ...  # type: List[List[Text]]


def to_wade_glides(pinyin: Text, **kwargs: Any) -> Text: ...



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pypinyin.style._tone_convert import (  # noqa
    to_normal,
    to_tone,
    to_tone2,
    to_tone3,
    to_initials,
    to_finals,
    to_finals_tone,
    to_finals_tone2,
    to_finals_tone3,
    tone_to_normal,
    tone_to_tone2,
    tone_to_tone3,
    tone2_to_normal,
    tone2_to_tone,
    tone2_to_tone3,
    tone3_to_normal,
    tone3_to_tone,
    tone3_to_tone2,
    # 向后兼容
    _improve_tone3,
    _get_number_from_pinyin,
    _v_to_u,
    _fix_v_u,
    _re_number,
)  # noqa



# -*- coding: utf-8 -*-
from typing import Any, List, Text, ByteString

SUPPORT_UCS4 = ...  # type: bool

PY2 = ...  # type: bool

subversion = ...  # type: List[Text]

text_type = ...  # type: Text
bytes_type = ...  # type: ByteString


def callable_check(obj: Any) -> bool: ...



from typing import Iterator
from typing import Text
from typing import Set


class Seg(object):
    """最大正向匹配分词

    :type prefix_set: PrefixSet
    :type no_non_phrases: bool
    """
    def __init__(self, prefix_set: PrefixSet, no_non_phrases: bool) -> None:
        self._no_non_phrases = ...  # type: bool
        self._prefix_set = ...  # type: PrefixSet
        ...

    def cut(self, text: Text) -> Iterator[Text]: ...

    def train(self, words: Iterator[Text]) -> None: ...


class PrefixSet(object):
    def __init__(self) -> None:
        self._set = ...  # type: Set[Text]
        ...

    def train(self, word_s: Iterator[Text]) -> None: ...

    def __contains__(self, key: Text) -> bool: ...


p_set = ...  # type: PrefixSet
seg = ...  # type: Seg


def retrain(seg_instance: Seg) -> None: ...



# -*- coding: utf-8 -*-
from typing import Any
from typing import Optional
from typing import Text
from typing import Tuple

from pypinyin.constants import Style

TStyle = Style


class NeutralToneWith5Mixin(object):
    NUMBER_TONE = ...  # type: Tuple[TStyle]
    NUMBER_AT_END = ...  # type: Tuple[TStyle]

    def post_convert_style(self, han: Text, orig_pinyin: Text,
                           converted_pinyin: Text, style: TStyle,
                           strict: bool, **kwargs: Any) -> Optional[Text]: ...



from typing import Any, Dict, Tuple, Text

# 俄语转换表
CYRILLIC_REPLACE = ...  # type: Tuple[Tuple[Any]]
CYRILLIC_TABLE = ...  # type: Dict[Text, Text]


class CyrillicfoConverter(object):
    def to_cyrillic(self, pinyin: Text, **kwargs: Any) -> Text: ...

    def to_cyrillic_first(self, pinyin: Text, **kwargs: Any) -> Text: ...

    def _pre_convert(self, pinyin: Text) -> Text: ...


converter = ...  # type: CyrillicfoConverter



# -*- coding: utf-8 -*-
"""TONE 相关的几个拼音风格实现:

Style.TONE
Style.TONE2
Style.TONE3
"""
from __future__ import unicode_literals

from pypinyin.constants import Style
from pypinyin.style import register
from pypinyin.style._constants import RE_TONE3
from pypinyin.style._utils import replace_symbol_to_number


class ToneConverter(object):
    def to_tone(self, pinyin, **kwargs):
        return pinyin

    def to_tone2(self, pinyin, **kwargs):
        # 用数字表示声调
        pinyin = replace_symbol_to_number(pinyin)
        return pinyin

    def to_tone3(self, pinyin, **kwargs):
        pinyin = self.to_tone2(pinyin, **kwargs)
        # 将声调数字移动到最后
        return RE_TONE3.sub(r'\1\3\2', pinyin)


converter = ToneConverter()
register(Style.TONE, func=converter.to_tone)
register(Style.TONE2, func=converter.to_tone2)
register(Style.TONE3, func=converter.to_tone3)



from typing import Any
from typing import List
from typing import Union
from typing import Callable
from typing import Optional
from typing import Text

from pypinyin.constants import Style


TStyle = Style
TErrors = Union[Callable[[Text], Text], Text]
TPinyinResult = List[List[Text]]
TErrorResult = Union[Text, List[Text], None]
TNoPinyinResult = Union[TPinyinResult, List[Text], Text, None]


class Converter(object):
    def convert(self, words: Text, style: TStyle, heteronym: bool,
                errors: TErrors, strict: bool = ...,
                **kwargs: Any) -> TPinyinResult: ...


class DefaultConverter(Converter):
    def __init__(self, **kwargs: Any) -> None: ...

    def convert(self, words: Text, style: TStyle, heteronym: bool,
                errors: TErrors, strict: bool = ...,
                **kwargs: Any) -> TPinyinResult: ...

    def pre_convert_style(self, han: Text, orig_pinyin: Text, style: TStyle,
                          strict: bool, **kwargs: Any) -> Optional[Text]: ...

    def convert_style(self, han: Text, orig_pinyin: Text, style: TStyle,
                      strict: bool, **kwargs: Any) -> Text: ...

    def post_convert_style(self, han: Text, orig_pinyin: Text,
                           converted_pinyin: Text, style: TStyle,
                           strict: bool, **kwargs: Any) -> Optional[Text]: ...

    def pre_handle_nopinyin(self, chars: Text, style: TStyle, heteronym: bool,
                            errors: TErrors, strict: bool
                            ) -> TNoPinyinResult: ...

    def handle_nopinyin(self, chars: Text, style: TStyle, heteronym: bool,
                        errors: TErrors, strict: bool, **kwargs: Any
                        ) -> TPinyinResult: ...

    def post_handle_nopinyin(self, chars: Text, style: Style, heteronym: bool,
                             errors: TErrors, strict: bool,
                             pinyin: TNoPinyinResult, **kwargs: Any
                             ) -> TNoPinyinResult: ...

    def post_pinyin(self, han: Text, heteronym: bool,
                    pinyin: TPinyinResult,
                    **kwargs: Any) -> Union[TPinyinResult, None]: ...

    def convert_styles(self, pinyin_list: TPinyinResult,
                       phrase: Text, style: TStyle, heteronym: bool,
                       errors: TErrors, strict: bool, **kwargs: Any,
                       ) -> TPinyinResult: ...

    def _phrase_pinyin(self, phrase: Text, style: TStyle, heteronym: bool,
                       errors: TErrors, strict: bool
                       ) -> TPinyinResult: ...

    def _single_pinyin(self, han: Text, style: TStyle, heteronym: bool,
                       errors: TErrors, strict: bool
                       ) -> TPinyinResult: ...

    def _convert_style(self, han: Text, pinyin: Text, style: TStyle,
                       strict: bool, default: Text, **kwargs: Any
                       ) -> Text: ...

    def _convert_nopinyin_chars(self, chars: Text, style: TStyle,
                                heteronym: bool, errors: TErrors,
                                strict: bool
                                ) -> TNoPinyinResult: ...


class UltimateConverter(DefaultConverter):
    def __init__(self, **kwargs: Any) -> None:
        self._tone_sandhi = ...
        self._neutral_tone_with_five = ...
        self._v_to_u = ...
        ...

    def convert(self, words: Text, style: TStyle, heteronym: bool,
                errors: TErrors, strict: bool = ...,
                **kwargs: Any) -> TPinyinResult: ...

    def pre_convert_style(self, han: Text, orig_pinyin: Text, style: TStyle,
                          strict: bool, **kwargs: Any) -> Optional[Text]: ...

    def convert_style(self, han: Text, orig_pinyin: Text, style: TStyle,
                      strict: bool, **kwargs: Any) -> Text: ...

    def post_convert_style(self, han: Text, orig_pinyin: Text,
                           converted_pinyin: Text, style: TStyle,
                           strict: bool, **kwargs: Any) -> Optional[Text]: ...

    def pre_handle_nopinyin(self, chars: Text, style: TStyle, heteronym: bool,
                            errors: TErrors, strict: bool
                            ) -> TNoPinyinResult: ...

    def handle_nopinyin(self, chars: Text, style: TStyle, heteronym: bool,
                        errors: TErrors, strict: bool, **kwargs: Any
                        ) -> TPinyinResult: ...

    def post_handle_nopinyin(self, chars: Text, style: Style, heteronym: bool,
                             errors: TErrors, strict: bool,
                             pinyin: TNoPinyinResult, **kwargs: Any
                             ) -> TNoPinyinResult: ...

    def post_pinyin(self, han: Text, heteronym: bool,
                    pinyin: TPinyinResult,
                    **kwargs: Any) -> Union[TPinyinResult, None]: ...

    def _phrase_pinyin(self, phrase: Text, style: TStyle, heteronym: bool,
                       errors: TErrors, strict: bool
                       ) -> TPinyinResult: ...

    def _single_pinyin(self, han: Text, style: TStyle, heteronym: bool,
                       errors: TErrors, strict: bool
                       ) -> TPinyinResult: ...

    def _convert_style(self, han: Text, pinyin: Text, style: TStyle,
                       strict: bool, default: Text, **kwargs: Any
                       ) -> Text: ...

    def _convert_nopinyin_chars(self, chars: Text, style: TStyle,
                                heteronym: bool, errors: TErrors,
                                strict: bool
                                ) -> TNoPinyinResult: ...



# -*- coding: utf-8 -*-
"""韵母相关拼音风格:

Style.FINALS
Style.FINALS_TONE
Style.FINALS_TONE2
Style.FINALS_TONE3
"""
from __future__ import unicode_literals

from pypinyin.constants import Style
from pypinyin.style import register
from pypinyin.style._tone_convert import (
    to_finals, to_finals_tone, to_finals_tone2, to_finals_tone3
)


class FinalsConverter(object):
    def to_finals(self, pinyin, **kwargs):
        """无声调韵母"""
        return to_finals(pinyin, strict=kwargs.get('strict', True))

    def to_finals_tone(self, pinyin, **kwargs):
        """声调在韵母头上"""
        return to_finals_tone(pinyin, strict=kwargs.get('strict', True))

    def to_finals_tone2(self, pinyin, **kwargs):
        """数字声调"""
        return to_finals_tone2(pinyin, strict=kwargs.get('strict', True))

    def to_finals_tone3(self, pinyin, **kwargs):
        """数字声调"""
        return to_finals_tone3(pinyin, strict=kwargs.get('strict', True))


converter = FinalsConverter()
register(Style.FINALS, func=converter.to_finals)
register(Style.FINALS_TONE, func=converter.to_finals_tone)
register(Style.FINALS_TONE2, func=converter.to_finals_tone2)
register(Style.FINALS_TONE3, func=converter.to_finals_tone3)



from typing import Any
from typing import List
from typing import Dict
from typing import Union
from typing import Callable
from typing import Optional
from typing import Text

from pypinyin.constants import Style
from pypinyin.converter import Converter


TStyle = Union[Style, Text]
TErrors = Union[Callable[[Text], Text], Text]
TPinyinResult = List[List[Text]]


def load_single_dict(pinyin_dict: Dict[int, Text],
                     style: str = ...) -> None: ...


def load_phrases_dict(phrases_dict: Dict[Text, List[List[Text]]],
                      style: str = ...
                      ) -> None: ...


def to_fixed(pinyin: Text, style: TStyle,
             strict: bool = ...) -> Text: ...


def _handle_nopinyin_char(chars: Text, errors: TErrors = ...
                          ) -> Optional[Text]: ...


def handle_nopinyin(chars: Text, errors: TErrors = ..., heteronym: bool = ...
                    ) -> List[List[Text]]: ...


def single_pinyin(han: Text, style: TStyle, heteronym: bool,
                  errors: TErrors = ...,
                  strict: bool = ...
                  ) -> List[List[Text]]: ...


def phrase_pinyin(phrase: Text,
                  style: TStyle,
                  heteronym: bool,
                  errors: TErrors = ...,
                  strict: bool = ...
                  ) -> List[List[Text]]: ...


def _pinyin(words: Text,
            style: TStyle,
            heteronym: bool,
            errors: TErrors,
            strict: bool = ...
            ) -> List[List[Text]]:...


def pinyin(hans: Union[List[Text], Text],
           style: TStyle = ...,
           heteronym: bool = ...,
           errors: TErrors = ...,
           strict: bool = ...,
           v_to_u: bool = ...,
           neutral_tone_with_five: bool = ...
           ) -> List[List[Text]]: ...


def slug(hans: Union[List[Text], Text],
         style: TStyle = ...,
         heteronym: bool = ...,
         separator: Text = ...,
         errors: TErrors = ...,
         strict: bool = ...
         ) -> Text: ...


def lazy_pinyin(hans: Union[List[Text], Text],
                style: TStyle = ...,
                errors: TErrors = ...,
                strict: bool = ...,
                v_to_u: bool = ...,
                neutral_tone_with_five: bool = ...,
                tone_sandhi: bool = ...
                ) -> List[Text]: ...


class Pinyin(object):

    def __init__(self, converter: Converter = ..., **kwargs: Any) -> None:
        self._converter = ...  # type: Converter

    def pinyin(self, hans: Union[List[Text], Text],
               style: TStyle = ...,
               heteronym: bool = ...,
               errors: TErrors = ...,
               strict: bool = ...,
               **kwargs: Any
               ) -> TPinyinResult: ...

    def lazy_pinyin(self, hans: Union[List[Text], Text],
                    style: TStyle = ...,
                    errors: TErrors = ...,
                    strict: bool = ...,
                    **kwargs: Any
                    ) -> List[Text]: ...

    def pre_seg(self, hans: Text,
                **kwargs: Any) -> Optional[List[Text]]: ...

    def post_seg(self, hans: Text, seg_data: List[Text],
                 **kwargs: Any) -> Optional[List[Text]]: ...

    def seg(self, hans: Text, **kwargs: Any) -> List[Text]: ...

    def get_seg(self, **kwargs: Any) -> Callable[[Text], List[Text]]: ...



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

"""
标调位置

    有 ɑ 不放过，

　　没 ɑ 找 o、e；

　　ɑ、o、e、i、u、ü

　　标调就按这顺序；

　　i、u 若是连在一起，

　　谁在后面就标谁。

http://www.hwjyw.com/resource/content/2010/06/04/8183.shtml
https://www.zhihu.com/question/23655297
https://github.com/mozillazg/python-pinyin/issues/160
http://www.pinyin.info/rules/where.html
"""

# TODO: 增加测试用例：使用 pinyin_dict 中的数据收集所有带声调拼音测试这个规则的正确性
#       1. 收集所有带声调拼音
#       2. 转换为数字声调拼音(tone3)，然后再转换为声调拼音
#       3. 比对转换后的声调拼音跟原始拼音，确保结果一致


def right_mark_index(pinyin_no_tone):
    # 'iou', 'uei', 'uen': 还原原始韵母后，还需要按照还原前的声调进行标记
    if 'iou' in pinyin_no_tone:
        return pinyin_no_tone.index('u')
    if 'uei' in pinyin_no_tone:
        return pinyin_no_tone.index('i')
    if 'uen' in pinyin_no_tone:
        return pinyin_no_tone.index('u')

    # 有 ɑ 不放过, 没 ɑ 找 o、e
    for c in ['a', 'o', 'e']:
        if c in pinyin_no_tone:
            return pinyin_no_tone.index(c) + len(c) - 1

    # i、u 若是连在一起，谁在后面就标谁
    for c in ['iu', 'ui']:
        if c in pinyin_no_tone:
            return pinyin_no_tone.index(c) + len(c) - 1

    # ɑ、o、e、i、u、ü
    for c in ['i', 'u', 'v', 'ü']:
        if c in pinyin_no_tone:
            return pinyin_no_tone.index(c) + len(c) - 1

    # n, m, ê
    for c in ['n', 'm', 'ê']:
        if c in pinyin_no_tone:
            return pinyin_no_tone.index(c) + len(c) - 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from itertools import chain

from pypinyin.compat import text_type
from pypinyin.constants import (
    PHRASES_DICT, PINYIN_DICT, Style, RE_HANS
)
from pypinyin.converter import DefaultConverter, UltimateConverter
from pypinyin.contrib.tone_sandhi import ToneSandhiMixin
from pypinyin.contrib.tone_convert import tone2_to_tone
from pypinyin.seg import mmseg
from pypinyin.seg.simpleseg import seg


def load_single_dict(pinyin_dict, style='default'):
    """载入用户自定义的单字拼音库

    :param pinyin_dict: 单字拼音库。比如： ``{0x963F: u"ā,ē"}``
    :param style: pinyin_dict 参数值的拼音库风格. 支持 'default', 'tone2'
    :type pinyin_dict: dict
    """
    if style == 'tone2':
        for k, v in pinyin_dict.items():
            v = tone2_to_tone(v)
            PINYIN_DICT[k] = v
    else:
        PINYIN_DICT.update(pinyin_dict)

    mmseg.retrain(mmseg.seg)


def load_phrases_dict(phrases_dict, style='default'):
    """载入用户自定义的词语拼音库

    :param phrases_dict: 词语拼音库。比如： ``{u"阿爸": [[u"ā"], [u"bà"]]}``
    :param style: phrases_dict 参数值的拼音库风格. 支持 'default', 'tone2'
    :type phrases_dict: dict
    """
    if style == 'tone2':
        for k, value in phrases_dict.items():
            v = [
                list(map(tone2_to_tone, pys))
                for pys in value
            ]
            PHRASES_DICT[k] = v
    else:
        PHRASES_DICT.update(phrases_dict)

    mmseg.retrain(mmseg.seg)


class Pinyin(object):

    def __init__(self, converter=None, **kwargs):
        self._converter = converter or DefaultConverter()

    def pinyin(self, hans, style=Style.TONE, heteronym=False,
               errors='default', strict=True, **kwargs):
        """将汉字转换为拼音，返回汉字的拼音列表。

        :param hans: 汉字字符串( ``'你好吗'`` )或列表( ``['你好', '吗']`` ).
                     可以使用自己喜爱的分词模块对字符串进行分词处理,
                     只需将经过分词处理的字符串列表传进来就可以了。
        :type hans: unicode 字符串或字符串列表
        :param style: 指定拼音风格，默认是 :py:attr:`~pypinyin.Style.TONE` 风格。
                      更多拼音风格详见 :class:`~pypinyin.Style`
        :param errors: 指定如何处理没有拼音的字符。详见 :ref:`handle_no_pinyin`

                       * ``'default'``: 保留原始字符
                       * ``'ignore'``: 忽略该字符
                       * ``'replace'``: 替换为去掉 ``\\u`` 的 unicode 编码字符串
                         (``'\\u90aa'`` => ``'90aa'``)
                       * callable 对象: 回调函数之类的可调用对象。

        :param heteronym: 是否启用多音字
        :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                       是否严格遵照《汉语拼音方案》来处理声母和韵母，
                       详见 :ref:`strict`
        :return: 拼音列表
        :rtype: list

        """
        # 对字符串进行分词处理
        if isinstance(hans, text_type):
            han_list = self.seg(hans)
        else:
            if isinstance(self._converter, UltimateConverter) or \
                    isinstance(self._converter, ToneSandhiMixin):
                han_list = []
                for h in hans:
                    if not RE_HANS.match(h):
                        han_list.extend(self.seg(h))
                    else:
                        han_list.append(h)
            else:
                han_list = chain(*(self.seg(x) for x in hans))

        pys = []
        for words in han_list:
            pys.extend(
                self._converter.convert(
                    words, style, heteronym, errors, strict=strict))
        return pys

    def lazy_pinyin(self, hans, style=Style.NORMAL,
                    errors='default', strict=True, **kwargs):
        """将汉字转换为拼音，返回不包含多音字结果的拼音列表.

        与 :py:func:`~pypinyin.pinyin` 的区别是每个汉字的拼音是个字符串，
        并且每个字只包含一个读音.

        :param hans: 汉字字符串( ``'你好吗'`` )或列表( ``['你好', '吗']`` ).
                 可以使用自己喜爱的分词模块对字符串进行分词处理,
                 只需将经过分词处理的字符串列表传进来就可以了。
        :type hans: unicode 字符串或字符串列表
        :param style: 指定拼音风格，默认是 :py:attr:`~pypinyin.Style.NORMAL` 风格。
                      更多拼音风格详见 :class:`~pypinyin.Style`。
        :param errors: 指定如何处理没有拼音的字符，详情请参考
                       :py:func:`~pypinyin.pinyin`
        :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                       是否严格遵照《汉语拼音方案》来处理声母和韵母，
                       详见 :ref:`strict`
        :return: 拼音列表(e.g. ``['zhong', 'guo', 'ren']``)
        :rtype: list

        """
        return list(
            chain(
                *self.pinyin(
                    hans, style=style, heteronym=False,
                    errors=errors, strict=strict)))

    def pre_seg(self, hans, **kwargs):
        """对字符串进行分词前将调用 ``pre_seg`` 方法对未分词的字符串做预处理。

        默认原样返回传入的 ``hans``。

        如果这个方法的返回值类型是 ``list``，表示返回的是一个分词后的结果，此时，
        ``seg`` 方法中将不再调用 ``seg_function`` 进行分词。

        :param hans: 分词前的字符串
        :return: ``None`` or ``list``
        """
        pass

    def seg(self, hans, **kwargs):
        """对汉字进行分词。

        分词前会调用 ``pre_seg`` 方法，分词后会调用 ``post_seg`` 方法。

        :param hans:
        :return:
        """
        pre_data = self.pre_seg(hans)
        if isinstance(pre_data, list):
            seg_data = pre_data
        else:
            seg_data = self.get_seg()(hans)

        post_data = self.post_seg(hans, seg_data)
        if isinstance(post_data, list):
            return post_data

        return seg_data

    def get_seg(self, **kwargs):
        """获取分词函数。

        :return: 分词函数
        """
        return seg

    def post_seg(self, hans, seg_data, **kwargs):
        """对字符串进行分词后将调用 ``post_seg`` 方法对分词后的结果做处理。

        默认原样返回传入的 ``seg_data``。

        如果这个方法的返回值类型是 ``list``，表示对分词结果做了二次处理，此时，
        ``seg`` 方法将以这个返回的数据作为返回值。

        :param hans: 分词前的字符串
        :param seg_data: 分词后的结果
        :type seg_data: list
        :return: ``None`` or ``list``
        """
        pass


_default_convert = DefaultConverter()
_default_pinyin = Pinyin(_default_convert)


def to_fixed(pinyin, style, strict=True):
    # 用于向后兼容，TODO: 废弃
    return _default_convert.convert_style(
        '', pinyin, style=style, strict=strict, default=pinyin)


_to_fixed = to_fixed


def handle_nopinyin(chars, errors='default', heteronym=True):
    # 用于向后兼容，TODO: 废弃
    return _default_convert.handle_nopinyin(
        chars, style=None, errors=errors, heteronym=heteronym, strict=True)


def single_pinyin(han, style, heteronym, errors='default', strict=True):
    # 用于向后兼容，TODO: 废弃
    return _default_convert._single_pinyin(
        han, style, heteronym, errors=errors, strict=strict)


def phrase_pinyin(phrase, style, heteronym, errors='default', strict=True):
    # 用于向后兼容，TODO: 废弃
    return _default_convert._phrase_pinyin(
        phrase, style, heteronym, errors=errors, strict=strict)


def pinyin(hans, style=Style.TONE, heteronym=False,
           errors='default', strict=True,
           v_to_u=False, neutral_tone_with_five=False):
    """将汉字转换为拼音，返回汉字的拼音列表。

    :param hans: 汉字字符串( ``'你好吗'`` )或列表( ``['你好', '吗']`` ).
                 可以使用自己喜爱的分词模块对字符串进行分词处理,
                 只需将经过分词处理的字符串列表传进来就可以了。
    :type hans: unicode 字符串或字符串列表
    :param style: 指定拼音风格，默认是 :py:attr:`~pypinyin.Style.TONE` 风格。
                  更多拼音风格详见 :class:`~pypinyin.Style`
    :param errors: 指定如何处理没有拼音的字符。详见 :ref:`handle_no_pinyin`

                   * ``'default'``: 保留原始字符
                   * ``'ignore'``: 忽略该字符
                   * ``'replace'``: 替换为去掉 ``\\u`` 的 unicode 编码字符串
                     (``'\\u90aa'`` => ``'90aa'``)
                   * callable 对象: 回调函数之类的可调用对象。

    :param heteronym: 是否启用多音字
    :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                   是否严格遵照《汉语拼音方案》来处理声母和韵母，
                   详见 :ref:`strict`
    :param v_to_u: 无声调相关拼音风格下的结果是否使用 ``ü`` 代替原来的 ``v``
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :type v_to_u: bool
    :param neutral_tone_with_five: 声调使用数字表示的相关拼音风格下的结果是否
                                   使用 5 标识轻声
    :type neutral_tone_with_five: bool
    :return: 拼音列表
    :rtype: list

    :raise AssertionError: 当传入的字符串不是 unicode 字符时会抛出这个异常

    Usage::

      >>> from pypinyin import pinyin, Style
      >>> import pypinyin
      >>> pinyin('中心')
      [['zhōng'], ['xīn']]
      >>> pinyin('中心', heteronym=True)  # 启用多音字模式
      [['zhōng', 'zhòng'], ['xīn']]
      >>> pinyin('中心', style=Style.FIRST_LETTER)  # 设置拼音风格
      [['z'], ['x']]
      >>> pinyin('中心', style=Style.TONE2)
      [['zho1ng'], ['xi1n']]
      >>> pinyin('中心', style=Style.CYRILLIC)
      [['чжун1'], ['синь1']]
      >>> pinyin('战略', v_to_u=True, style=Style.NORMAL)
      [['zhan'], ['lüe']]
      >>> pinyin('衣裳', style=Style.TONE3, neutral_tone_with_five=True)
      [['yi1'], ['shang5']]
    """
    _pinyin = Pinyin(UltimateConverter(
        v_to_u=v_to_u, neutral_tone_with_five=neutral_tone_with_five))
    return _pinyin.pinyin(
        hans, style=style, heteronym=heteronym, errors=errors, strict=strict)


def slug(hans, style=Style.NORMAL, heteronym=False, separator='-',
         errors='default', strict=True):
    """将汉字转换为拼音，然后生成 slug 字符串.

    :param hans: 汉字字符串( ``'你好吗'`` )或列表( ``['你好', '吗']`` ).
                 可以使用自己喜爱的分词模块对字符串进行分词处理,
                 只需将经过分词处理的字符串列表传进来就可以了。
    :type hans: unicode 字符串或字符串列表
    :param style: 指定拼音风格，默认是 :py:attr:`~pypinyin.Style.NORMAL` 风格。
                  更多拼音风格详见 :class:`~pypinyin.Style`
    :param heteronym: 是否启用多音字
    :param separator: 两个拼音间的分隔符/连接符
    :param errors: 指定如何处理没有拼音的字符，详情请参考
                   :py:func:`~pypinyin.pinyin`
    :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                   是否严格遵照《汉语拼音方案》来处理声母和韵母，
                   详见 :ref:`strict`
    :return: slug 字符串.

    :raise AssertionError: 当传入的字符串不是 unicode 字符时会抛出这个异常

    ::

      >>> import pypinyin
      >>> from pypinyin import Style
      >>> pypinyin.slug('中国人')
      'zhong-guo-ren'
      >>> pypinyin.slug('中国人', separator=' ')
      'zhong guo ren'
      >>> pypinyin.slug('中国人', style=Style.FIRST_LETTER)
      'z-g-r'
      >>> pypinyin.slug('中国人', style=Style.CYRILLIC)
      'чжун1-го2-жэнь2'
    """
    return separator.join(
        chain(
            *_default_pinyin.pinyin(
                hans, style=style, heteronym=heteronym,
                errors=errors, strict=strict
            )
        )
    )


def lazy_pinyin(hans, style=Style.NORMAL, errors='default', strict=True,
                v_to_u=False, neutral_tone_with_five=False, tone_sandhi=False):
    """将汉字转换为拼音，返回不包含多音字结果的拼音列表.

    与 :py:func:`~pypinyin.pinyin` 的区别是返回的拼音是个字符串，
    并且每个字只包含一个读音.

    :param hans: 汉字字符串( ``'你好吗'`` )或列表( ``['你好', '吗']`` ).
                 可以使用自己喜爱的分词模块对字符串进行分词处理,
                 只需将经过分词处理的字符串列表传进来就可以了。
    :type hans: unicode 字符串或字符串列表
    :param style: 指定拼音风格，默认是 :py:attr:`~pypinyin.Style.NORMAL` 风格。
                  更多拼音风格详见 :class:`~pypinyin.Style`。
    :param errors: 指定如何处理没有拼音的字符，详情请参考
                   :py:func:`~pypinyin.pinyin`
    :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                   是否严格遵照《汉语拼音方案》来处理声母和韵母，
                   详见 :ref:`strict`
    :param v_to_u: 无声调相关拼音风格下的结果是否使用 ``ü`` 代替原来的 ``v``
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :type v_to_u: bool
    :param neutral_tone_with_five: 声调使用数字表示的相关拼音风格下的结果是否
                                   使用 5 标识轻声
    :type neutral_tone_with_five: bool
    :param tone_sandhi: 是否按照声调 `变调规则 <https://en.wikipedia.org/wiki/Standard_Chinese_phonology#Tone_sandhi>`__
                        对拼音进行处理
                        （使用预先通过分词库进行过分词后的结果作为 ``hans``
                        参数的值效果会更好，因为变调效果依赖分词效果）
    :type tone_sandhi: bool
    :return: 拼音列表(e.g. ``['zhong', 'guo', 'ren']``)
    :rtype: list

    :raise AssertionError: 当传入的字符串不是 unicode 字符时会抛出这个异常

    Usage::

      >>> from pypinyin import lazy_pinyin, Style
      >>> import pypinyin
      >>> lazy_pinyin('中心')
      ['zhong', 'xin']
      >>> lazy_pinyin('中心', style=Style.TONE)
      ['zhōng', 'xīn']
      >>> lazy_pinyin('中心', style=Style.FIRST_LETTER)
      ['z', 'x']
      >>> lazy_pinyin('中心', style=Style.TONE2)
      ['zho1ng', 'xi1n']
      >>> lazy_pinyin('中心', style=Style.CYRILLIC)
      ['чжун1', 'синь1']
      >>> lazy_pinyin('战略', v_to_u=True)
      ['zhan', 'lüe']
      >>> lazy_pinyin('衣裳', style=Style.TONE3, neutral_tone_with_five=True)
      ['yi1', 'shang5']
      >>> lazy_pinyin('你好', style=Style.TONE2, tone_sandhi=True)
      ['ni2', 'ha3o']
    """  # noqa
    _pinyin = Pinyin(UltimateConverter(
        v_to_u=v_to_u, neutral_tone_with_five=neutral_tone_with_five,
        tone_sandhi=tone_sandhi))
    return _pinyin.lazy_pinyin(
        hans, style=style, errors=errors, strict=strict)



# -*- coding: utf-8 -*-
"""CYRILLIC 相关的几个拼音风格实现:

Style.CYRILLIC
Style.CYRILLIC_FIRST
"""
from __future__ import unicode_literals
import re

from pypinyin.constants import Style
from pypinyin.style import register
from pypinyin.style._constants import RE_TONE3
from pypinyin.style._utils import replace_symbol_to_number

# 俄语转换表
CYRILLIC_REPLACE = (
    (re.compile(r'ong'), 'ung'),
    (re.compile(r'([zcs])i'), '\\1U'),
    (re.compile(r'([xqj])u'), '\\1v'),
    (re.compile(r'^wu(.?)$'), 'u\\1'),
    (re.compile(r'(.+)r(.?)$'), '\\1R\\2'),
    (re.compile(r'^zh'), 'Cr'),
    (re.compile(r'^ch'), 'C'),
    (re.compile(r'^j'), 'qZ'),
    (re.compile(r'^z'), 'qZ'),
    (re.compile(r'^x'), 's'),
    (re.compile(r'^sh'), 'S'),
    (re.compile(r'([^CSdst])uo'), '\\1o'),
    (re.compile(r'^y(.*)$'), 'I\\1'),
    (re.compile(r'Iai'), 'AI'),
    (re.compile(r'Ia'), 'A'),
    (re.compile(r'Ie'), 'E'),
    (re.compile(r'Ii'), 'i'),
    (re.compile(r'Iou'), 'V'),
    (re.compile(r'Iu'), 'v'),
    (re.compile(r'(.v)(\d?)$'), '\\1I\\2'),
    (re.compile(r'Io'), 'O'),
    (re.compile(r'iu'), 'v'),
    (re.compile(r'ie'), 'E'),
    (re.compile(r'hui'), 'huei'),
    (re.compile(r'ui'), 'uI'),
    (re.compile(r'ai'), 'aI'),
    (re.compile(r'ei'), 'eI'),
    (re.compile(r'ia'), 'A'),
    (re.compile(r'(.*[^h])n([^g]?)$'), '\\1nM\\2'),
    (re.compile(r'(.*[^h])ng(.?)$'), '\\1n\\2'),
    (re.compile(r'^v(\d?$)'), 'vI'),
)
CYRILLIC_TABLE = dict(zip(
    u'abwgdEOrZiIklmnopRstufhqcCSHTMUevAV',
    u'абвгдеёжзийклмнопрстуфхццчшщъьыэюяю'
))


class CyrillicfoConverter(object):
    def to_cyrillic(self, pinyin, **kwargs):
        pinyin = self._pre_convert(pinyin)
        # 查表替换成注音
        for find_re, replace in CYRILLIC_REPLACE:
            pinyin = find_re.sub(replace, pinyin)
        pinyin = ''.join(CYRILLIC_TABLE.get(x, x) for x in pinyin)
        return pinyin

    def to_cyrillic_first(self, pinyin, **kwargs):
        pinyin = self.to_cyrillic(pinyin, **kwargs)
        return pinyin[0]

    def _pre_convert(self, pinyin):
        # 用数字表示声调
        pinyin = replace_symbol_to_number(pinyin)
        # 将声调数字移动到最后
        return RE_TONE3.sub(r'\1\3\2', pinyin)


converter = CyrillicfoConverter()
register(Style.CYRILLIC, func=converter.to_cyrillic)
register(Style.CYRILLIC_FIRST, func=converter.to_cyrillic_first)



from typing import List, Text


def _replace_tone2_style_dict_to_default(string: Text) -> Text: ...


def _remove_dup_items(lst: List[Text]) -> List[Text]: ...


def _remove_dup_and_empty(lst_list: List[List[Text]]) -> List[List[Text]]: ...



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class V2UMixin(object):
    """无声调相关拼音风格下的结果使用 ``ü`` 代替原来的 ``v``

    使用方法::

        from pypinyin import lazy_pinyin, Style
        from pypinyin.contrib.uv import V2UMixin
        from pypinyin.converter import DefaultConverter
        from pypinyin.core import Pinyin

        # 原来的结果中会使用 ``v`` 表示 ``ü``
        print(lazy_pinyin('战略'))
        # 输出：['zhan', 'lve']


        class MyConverter(V2UMixin, DefaultConverter):
            pass

        my_pinyin = Pinyin(MyConverter())
        pinyin = my_pinyin.pinyin
        lazy_pinyin = my_pinyin.lazy_pinyin

        #  新的结果中使用 ``ü`` 代替原来的 ``v``
        print(lazy_pinyin('战略'))
        # 输出: ['zhan', 'lüe']

        print(pinyin('战略', style=Style.NORMAL))
        # 输出：[['zhan'], ['lüe']]


    """

    def post_convert_style(self, han, orig_pinyin, converted_pinyin,
                           style, strict, **kwargs):
        pre_data = super(V2UMixin, self).post_convert_style(
            han, orig_pinyin, converted_pinyin, style, strict, **kwargs)

        if pre_data is not None:
            converted_pinyin = pre_data

        return converted_pinyin.replace('v', 'ü')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from pypinyin import Style
from pypinyin.contrib._tone_rule import right_mark_index


_re_number = re.compile(r'\d')


class NeutralToneWith5Mixin(object):
    """声调使用数字表示的相关拼音风格下的结果使用 5 标识轻声。

    使用方法::

        from pypinyin import lazy_pinyin, Style
        from pypinyin.contrib.neutral_tone import NeutralToneWith5Mixin
        from pypinyin.converter import DefaultConverter
        from pypinyin.core import Pinyin

        # 原来的结果中不会标识轻声
        print(lazy_pinyin('好了', style=Style.TONE2))
        # 输出: ['ha3o', 'le']


        class MyConverter(NeutralToneWith5Mixin, DefaultConverter):
            pass

        my_pinyin = Pinyin(MyConverter())
        pinyin = my_pinyin.pinyin
        lazy_pinyin = my_pinyin.lazy_pinyin

        #  新的结果中使用 ``5`` 标识轻声
        print(lazy_pinyin('好了', style=Style.TONE2))
        # 输出: ['ha3o', 'le5']

        print(pinyin('好了', style=Style.TONE2))
        # 输出：[['ha3o'], ['le5']]


    """

    NUMBER_TONE = (Style.TONE2, Style.TONE3, Style.FINALS_TONE2,
                   Style.FINALS_TONE3)
    NUMBER_AT_END = (Style.TONE3, Style.FINALS_TONE3)

    def post_convert_style(self, han, orig_pinyin, converted_pinyin,
                           style, strict, **kwargs):
        pre_data = super(NeutralToneWith5Mixin, self).post_convert_style(
            han, orig_pinyin, converted_pinyin, style, strict, **kwargs)

        if style not in self.NUMBER_TONE:
            return pre_data

        if pre_data is not None:
            converted_pinyin = pre_data
        if not converted_pinyin:    # 空字符串
            return converted_pinyin
        # 有声调，跳过
        if _re_number.search(converted_pinyin):
            return converted_pinyin

        if style in self.NUMBER_AT_END:
            return '{}5'.format(converted_pinyin)

        # 找到应该在哪个字母上标声调
        mark_index = right_mark_index(converted_pinyin)
        before = converted_pinyin[:mark_index + 1]
        after = converted_pinyin[mark_index + 1:]

        return '{}5{}'.format(before, after)



# -*- coding: utf-8 -*-
from typing import Any
from typing import Optional
from typing import Text
from typing import Tuple

_re_number = ...  # type: Any

def to_normal(pinyin: Text, v_to_u: bool = ...) -> Text: ...

def to_tone(pinyin: Text) -> Text: ...

def to_tone2(pinyin: Text, v_to_u: bool = ..., neutral_tone_with_five: bool = ...) -> Text: ...

def to_tone3(pinyin: Text, v_to_u: bool = ..., neutral_tone_with_five: bool = ...) -> Text: ...

def to_initials(pinyin: Text, strict: bool = ...) -> Text: ...

def to_finals(pinyin: Text, strict: bool = ..., v_to_u: bool = ...) -> Text: ...

def to_finals_tone(pinyin: Text, strict: bool = ...) -> Text: ...

def to_finals_tone2(pinyin: Text, strict: bool = ..., v_to_u: bool = ..., neutral_tone_with_five: bool = ...) -> Text: ...

def to_finals_tone3(pinyin: Text, strict: bool = ..., v_to_u: bool = ..., neutral_tone_with_five: bool = ...) -> Text: ...

def tone_to_normal(tone: Text, v_to_u: bool = ...) -> Text: ...

def tone_to_tone2(tone: Text, v_to_u: bool = ..., neutral_tone_with_five: bool = ...) -> Text: ...

def tone_to_tone3(tone: Text, v_to_u: bool = ..., neutral_tone_with_five: bool = ...) -> Text: ...

def tone2_to_normal(tone2: Text, v_to_u: bool = ...) -> Text: ...

def tone2_to_tone(tone2: Text,) -> Text: ...

def tone2_to_tone3(tone2: Text, v_to_u: bool = ...) -> Text: ...

def tone3_to_normal(tone3: Text, v_to_u: bool = ...) -> Text: ...

def tone3_to_tone(tone3: Text) -> Text: ...

def tone3_to_tone2(tone3: Text, v_to_u: bool = ...) -> Text: ...

def _improve_tone3(tone3: Text, neutral_tone_with_five: bool = ...) -> Text: ...

def _get_number_from_pinyin(pinyin: Text) -> Optional[int]: ...

def _v_to_u(pinyin: Text, replace: bool = ...) -> Text: ...

def _fix_v_u(origin_py: Text, new_py: Text, v_to_u: bool) -> Text: ...



from typing import Any
from typing import List
from typing import Union
from typing import Callable
from typing import Optional
from typing import Text

from pypinyin.constants import Style


TStyle = Style
TErrors = Union[Callable[[Text], Text], Text]
TPinyinResult = List[List[Text]]
TErrorResult = Union[Text, List[Text], None]
TNoPinyinResult = Union[TPinyinResult, List[Text], Text, None]

class ToneSandhiMixin(object):

    def post_pinyin(self, han: Text, heteronym: bool,
                    pinyin: TPinyinResult,
                    **kwargs: Any) -> Union[TPinyinResult, None]: ...



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import re

from pypinyin import phonetic_symbol

# 声母表
_INITIALS = [
    'b',
    'p',
    'm',
    'f',
    'd',
    't',
    'n',
    'l',
    'g',
    'k',
    'h',
    'j',
    'q',
    'x',
    'zh',
    'ch',
    'sh',
    'r',
    'z',
    'c',
    's',
]
# 声母表, 把 y, w 也当作声母
_INITIALS_NOT_STRICT = _INITIALS + ['y', 'w']

# 韵母表
_FINALS = [
    'i',
    'u',
    'ü',
    'a',
    'ia',
    'ua',
    'o',
    'uo',
    'e',
    'ie',
    'üe',
    'ai',
    'uai',
    'ei',
    'uei',
    'ao',
    'iao',
    'ou',
    'iou',
    'an',
    'ian',
    'uan',
    'üan',
    'en',
    'in',
    'uen',
    'ün',
    'ang',
    'iang',
    'uang',
    'eng',
    'ing',
    'ueng',
    'ong',
    'iong',

    'er',
    'ê',
]

# 带声调字符与数字表示声调的对应关系
PHONETIC_SYMBOL_DICT = phonetic_symbol.phonetic_symbol.copy()
PHONETIC_SYMBOL_DICT_KEY_LENGTH_NOT_ONE = dict(
    (k, v)
    for k, v in PHONETIC_SYMBOL_DICT.items()
    if len(k) > 1
)
# 匹配带声调字符的正则表达式
RE_PHONETIC_SYMBOL = re.compile(
    r'[{0}]'.format(
        re.escape(''.join(x for x in PHONETIC_SYMBOL_DICT if len(x) == 1))
    )
)

# 匹配使用数字标识声调的字符的正则表达式
RE_TONE2 = re.compile(r'([aeoiuvnmê])([1-5])$')
# 匹配 TONE2 中标识韵母声调的正则表达式
RE_TONE3 = re.compile(r'^([a-zê]+)([1-5])([a-zê]*)$')

# 匹配单个数字
RE_NUMBER = re.compile(r'\d')



from argparse import ArgumentParser
from typing import Union, Text, ByteString


class NullWriter(object):
    def write(self, string: Union[Text, ByteString]) -> None: ...


def get_parser() -> ArgumentParser: ...


def main() -> None: ...



from typing import Any, Text


class OthersConverter(object):
    def to_normal(self, pinyin: Text, **kwargs: Any) -> Text: ...

    def to_first_letter(self, pinyin: Text, **kwargs: Any) -> Text: ...


converter = ...  # type: OthersConverter



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import sys

SUPPORT_UCS4 = len('\U00020000') == 1

PY2 = sys.version_info < (3, 0)

subversion = getattr(sys, 'subversion', [''])
# 这些 Python 实现虽然是 Python 2 但字符串的行为跟 Python 3 是一样的
if subversion[0] in (
    'IronPython',
):
    PY2 = False  # pragma: no cover

if not PY2:
    text_type = str
    bytes_type = bytes
else:
    text_type = unicode  # noqa
    bytes_type = str

try:
    callable_check = callable  # noqa
except NameError:
    def callable_check(obj):
        return hasattr(obj, '__call__')



from enum import IntEnum, unique
from typing import Dict, List, Any, Text

PHRASES_DICT = ...  # type: Dict[Text, List[List[Text]]]

PINYIN_DICT = ...  # type: Dict[int, Text]

RE_TONE2 = ...  # type: Any

RE_HANS = ...  # type: Any


@unique
class Style(IntEnum):

    NORMAL = ...

    TONE = ...

    TONE2 = ...

    TONE3 = ...

    INITIALS = ...

    FIRST_LETTER = ...

    FINALS = ...

    FINALS_TONE = ...

    FINALS_TONE2 = ...

    FINALS_TONE3 = ...

    BOPOMOFO = ...

    BOPOMOFO_FIRST = ...

    CYRILLIC = ...

    CYRILLIC_FIRST = ...

    WADEGILES = ...


NORMAL = ...  # type: Style
STYLE_NORMAL = ...  # type: Style
TONE = ...  # type: Style
STYLE_TONE = ...  # type: Style
TONE2  = ...  # type: Style
STYLE_TONE2 = ...  # type: Style
TONE3 = ...  # type: Style
STYLE_TONE3 = ...  # type: Style
INITIALS = ...  # type: Style
STYLE_INITIALS = ...  # type: Style
FIRST_LETTER  = ...  # type: Style
STYLE_FIRST_LETTER = ...  # type: Style
FINALS = ...  # type: Style
STYLE_FINALS = ...  # type: Style
FINALS_TONE  = ...  # type: Style
STYLE_FINALS_TONE = ...  # type: Style
FINALS_TONE2 = ...  # type: Style
STYLE_FINALS_TONE2 = ...  # type: Style
FINALS_TONE3 = ...  # type: Style
STYLE_FINALS_TONE3 = ...  # type: Style
BOPOMOFO = ...  # type: Style
STYLE_BOPOMOFO = ...  # type: Style
BOPOMOFO_FIRST = ...  # type: Style
STYLE_BOPOMOFO_FIRST = ...  # type: Style
CYRILLIC = ...  # type: Style
STYLE_CYRILLIC = ...  # type: Style
CYRILLIC_FIRST = ...  # type: Style
STYLE_CYRILLIC_FIRST = ...  # type: Style



# -*- coding: utf-8 -*-

from typing import Any
from typing import Optional
from typing import Text

from pypinyin.constants import Style

TStyle = Style


class V2UMixin(object):

    def post_convert_style(self, han: Text, orig_pinyin: Text,
                           converted_pinyin: Text, style: TStyle,
                           strict: bool, **kwargs: Any) -> Optional[Text]: ...



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

# 用于向后兼容，TODO: 废弃
from pypinyin.seg.simpleseg import simple_seg  # noqa
from pypinyin.style._tone_convert import tone2_to_tone


# 用于向后兼容，TODO: 废弃
def _replace_tone2_style_dict_to_default(string):
    return tone2_to_tone(string)


def _remove_dup_items(lst, remove_empty=False):
    new_lst = []
    for item in lst:
        if remove_empty and not item:
            continue
        if item not in new_lst:
            new_lst.append(item)
    return new_lst


def _remove_dup_and_empty(lst_list):
    new_lst_list = []
    for lst in lst_list:
        lst = _remove_dup_items(lst, remove_empty=True)
        if lst:
            new_lst_list.append(lst)
        else:
            new_lst_list.append([''])

    return new_lst_list



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from pypinyin.standard import convert_finals
from pypinyin.style._constants import (
    _INITIALS, _INITIALS_NOT_STRICT, _FINALS,
    RE_PHONETIC_SYMBOL, PHONETIC_SYMBOL_DICT,
    PHONETIC_SYMBOL_DICT_KEY_LENGTH_NOT_ONE,
    RE_NUMBER
)


def get_initials(pinyin, strict):
    """获取单个拼音中的声母.

    :param pinyin: 单个拼音
    :type pinyin: unicode
    :param strict: 是否严格遵照《汉语拼音方案》来处理声母和韵母
    :return: 声母
    :rtype: unicode
    """
    if strict:
        _initials = _INITIALS
    else:
        _initials = _INITIALS_NOT_STRICT

    for i in _initials:
        if pinyin.startswith(i):
            return i
    return ''


def get_finals(pinyin, strict):
    """获取单个拼音中的韵母.

    :param pinyin: 单个拼音，无声调拼音
    :type pinyin: unicode
    :param strict: 是否严格遵照《汉语拼音方案》来处理声母和韵母
    :return: 韵母
    :rtype: unicode
    """
    if strict:
        pinyin = convert_finals(pinyin)

    initials = get_initials(pinyin, strict=strict) or ''

    # 按声母分割，剩下的就是韵母
    finals = pinyin[len(initials):]

    # 处理既没有声母也没有韵母的情况
    if strict and finals not in _FINALS:
        # 处理 y, w 导致误判的问题，比如 yo
        initials = get_initials(pinyin, strict=False)
        finals = pinyin[len(initials):]
        if finals in _FINALS:
            return finals
        return ''

    # ń, ḿ
    if not finals and not strict:
        return pinyin

    return finals


def replace_symbol_to_number(pinyin):
    """把声调替换为数字"""
    def _replace(match):
        symbol = match.group(0)  # 带声调的字符
        # 返回使用数字标识声调的字符
        return PHONETIC_SYMBOL_DICT[symbol]

    # 替换拼音中的带声调字符
    value = RE_PHONETIC_SYMBOL.sub(_replace, pinyin)
    for symbol, to in PHONETIC_SYMBOL_DICT_KEY_LENGTH_NOT_ONE.items():
        value = value.replace(symbol, to)

    return value


def replace_symbol_to_no_symbol(pinyin):
    """把带声调字符替换为没有声调的字符"""
    value = replace_symbol_to_number(pinyin)
    return RE_NUMBER.sub('', value)


def has_finals(pinyin):
    """判断是否有韵母"""
    # 鼻音: 'm̄', 'ḿ', 'm̀', 'ń', 'ň', 'ǹ ' 没有韵母
    for symbol in ['m̄', 'ḿ', 'm̀', 'ń', 'ň', 'ǹ']:
        if symbol in pinyin:
            return False

    return True



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from copy import deepcopy

from pypinyin.compat import text_type, callable_check
from pypinyin.constants import (
    PHRASES_DICT, PINYIN_DICT,
    RE_HANS
)
from pypinyin.contrib.uv import V2UMixin
from pypinyin.contrib.neutral_tone import NeutralToneWith5Mixin
from pypinyin.contrib.tone_sandhi import ToneSandhiMixin
from pypinyin.utils import _remove_dup_and_empty
from pypinyin.style import auto_discover
from pypinyin.style import convert as convert_style

auto_discover()


class Converter(object):

    def convert(self, words, style, heteronym, errors, strict, **kwargs):
        # TODO: use ``abc`` module
        raise NotImplementedError  # pragma: no cover


class DefaultConverter(Converter):
    def __init__(self, **kwargs):
        pass

    def convert(self, words, style, heteronym, errors, strict, **kwargs):
        """根据参数把汉字转成相应风格的拼音结果。

        :param words: 汉字字符串
        :type words: unicode
        :param style: 拼音风格
        :param heteronym: 是否启用多音字
        :type heteronym: bool
        :param errors: 如果处理没有拼音的字符
        :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                       是否严格遵照《汉语拼音方案》来处理声母和韵母，
                       详见 :ref:`strict`
        :type strict: bool
        :return: 按风格转换后的拼音结果
        :rtype: list

        """
        pys = []
        # 初步过滤没有拼音的字符
        if RE_HANS.match(words):
            pys = self._phrase_pinyin(words, style=style, heteronym=heteronym,
                                      errors=errors, strict=strict)
            post_data = self.post_pinyin(words, heteronym, pys)
            if post_data is not None:
                pys = post_data

            pys = self.convert_styles(
                pys, words, style, heteronym, errors, strict)

        else:
            py = self.handle_nopinyin(words, style=style, errors=errors,
                                      heteronym=heteronym, strict=strict)
            if py:
                pys.extend(py)

        return _remove_dup_and_empty(pys)

    def pre_convert_style(self, han, orig_pinyin, style, strict, **kwargs):
        """在把原始带声调的拼音按拼音风格转换前会调用 ``pre_convert_style`` 方法。

        如果返回值不为 ``None`` 会使用返回的结果代替 ``orig_pinyin``
        来进行后面的风格转换。

        :param han: 要处理的汉字
        :param orig_pinyin: 汉字对应的原始带声调拼音
        :param style: 要转换的拼音风格
        :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                       是否严格遵照《汉语拼音方案》来处理声母和韵母，
                       详见 :ref:`strict`
        :param kwargs: 其他关键字参数，暂时无用，用于以后扩展新的参数。
        :return: ``None`` 或代替 ``orig_pinyin`` 参与拼音风格转换的拼音字符串。

        """
        pass

    def convert_style(self, han, orig_pinyin, style, strict, **kwargs):
        """按 ``style`` 的值对 ``orig_pinyin`` 进行处理，返回处理后的拼音

        转换风格前会调用 ``pre_convert_style`` 方法，
        转换后会调用 ``post_convert_style`` 方法。

        :param han: 要处理的单个汉字
        :param orig_pinyin: 汉字对应的原始带声调拼音
        :param style: 拼音风格
        :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                       是否严格遵照《汉语拼音方案》来处理声母和韵母，
                       详见 :ref:`strict`
        :param kwargs: 其他关键字参数，暂时无用，用于以后扩展新的参数。
        :return: 按拼音风格转换处理后的拼音

        """
        pre_data = self.pre_convert_style(
            han, orig_pinyin, style=style, strict=strict)
        if pre_data is not None:
            pinyin = pre_data
        else:
            pinyin = orig_pinyin

        converted_pinyin = self._convert_style(
            han, pinyin, style=style, strict=strict, default=pinyin)

        post_data = self.post_convert_style(
            han, pinyin, converted_pinyin, style=style, strict=strict)
        if post_data is None:
            post_data = converted_pinyin

        return post_data

    def post_convert_style(self, han, orig_pinyin, converted_pinyin,
                           style, strict, **kwargs):
        """在把原始带声调的拼音按拼音风格转换前会调用 ``pre_convert_style`` 方法。

        如果返回值不为 ``None`` 会使用返回的结果代替 ``converted_pinyin``
        作为拼音风格转换后的最终拼音结果。

        :param han: 要处理的汉字
        :param orig_pinyin: 汉字对应的原始带声调拼音
        :param converted_pinyin: 按拼音风格转换处理后的拼音
        :param style: 要转换的拼音风格
        :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                       是否严格遵照《汉语拼音方案》来处理声母和韵母，
                       详见 :ref:`strict`
        :param kwargs: 其他关键字参数，暂时无用，用于以后扩展新的参数。
        :return: ``None`` 或代替 ``converted_pinyin`` 作为拼音风格转换后的拼音结果。

        """
        pass

    def pre_handle_nopinyin(self, chars, style, heteronym, errors,
                            strict, **kwargs):
        """处理没有拼音的字符串前会调用 ``pre_handle_nopinyin`` 方法。

        如果返回值不为 ``None`` 会使用返回的结果作为处理没有拼音字符串的结果，
        不再使用内置方法进行处理。

        :param chars: 待处理的没有拼音的字符串
        :param errors: 如何处理
        :param heteronym: 是否需要处理多音字
        :param kwargs: 其他关键字参数，暂时无用，用于以后扩展新的参数。
        :return: ``None`` 或代替 ``chars`` 参与拼音风格转换的拼音字符串
                  或拼音结果 list。

        """
        pass

    def handle_nopinyin(self, chars, style, heteronym, errors,
                        strict, **kwargs):
        """处理没有拼音的字符串。

        处理前会调用 ``pre_handle_nopinyin`` 方法，
        处理后会调用 ``post_handle_nopinyin`` 方法。

        :param chars: 待处理的没有拼音的字符串
        :param style: 拼音风格
        :param errors: 如何处理
        :param heteronym: 是否需要处理多音字
        :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                       是否严格遵照《汉语拼音方案》来处理声母和韵母，
                       详见 :ref:`strict`
        :return: 处理后的拼音结果，如果为 ``None`` 或空 list 表示忽略这个字符串.
        :rtype: list
        """
        pre_data = self.pre_handle_nopinyin(
            chars, style, errors=errors, heteronym=heteronym, strict=strict)

        if pre_data is not None:
            py = pre_data
        else:
            pre_data = chars
            py = self._convert_nopinyin_chars(
                pre_data, style, errors=errors,
                heteronym=heteronym, strict=strict)

        post_data = self.post_handle_nopinyin(
            chars, style, errors=errors, heteronym=heteronym, strict=strict,
            pinyin=py)
        if post_data is not None:
            py = post_data

        if not py:
            return []
        if isinstance(py, list):
            # 包含多音字信息
            if isinstance(py[0], list):
                if heteronym:
                    return py
                # [[a, b], [c, d]]
                # [[a], [c]]
                return [[x[0]] for x in py]

            return [[i] for i in py]
        else:
            return [[py]]

    def post_handle_nopinyin(self, chars, style, heteronym,
                             errors, strict,
                             pinyin, **kwargs):
        """处理完没有拼音的字符串后会调用 ``post_handle_nopinyin`` 方法。

        如果返回值不为 ``None`` 会使用返回的结果作为处理没有拼音的字符串的结果。

        :param chars: 待处理的没有拼音的字符串
        :param errors: 如何处理
        :param heteronym: 是否需要处理多音字
        :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                       是否严格遵照《汉语拼音方案》来处理声母和韵母，
                       详见 :ref:`strict`
        :param pinyin: 处理后的拼音信息，值为空 list 或包含拼音信息的 list
        :param kwargs: 其他关键字参数，暂时无用，用于以后扩展新的参数。
        :return: ``None`` 或代替 ``pinyin`` 做为处理结果。

        """
        pass

    def post_pinyin(self, han, heteronym, pinyin, **kwargs):
        """找到汉字对应的拼音后，会调用 ``post_pinyin`` 方法。

        如果返回值不为 ``None`` 会使用返回的结果作为 han 的拼音数据。

        :param han: 单个汉字或者词语
        :param heteronym: 是否需要处理多音字
        :param pinyin: 单个汉字的拼音数据或词语的拼音数据 list
        :type pinyin: list
        :param kwargs: 其他关键字参数，暂时无用，用于以后扩展新的参数。
        :return: ``None`` 或代替 ``pinyin`` 作为 han 的拼音 list。

        """
        pass

    def _phrase_pinyin(self, phrase, style, heteronym, errors, strict):
        """词语拼音转换.

        :param phrase: 词语
        :param errors: 指定如何处理没有拼音的字符
        :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                       是否严格遵照《汉语拼音方案》来处理声母和韵母，
                       详见 :ref:`strict`
        :return: 拼音列表
        :rtype: list
        """
        pinyin_list = []
        if phrase in PHRASES_DICT:
            pinyin_list = deepcopy(PHRASES_DICT[phrase])
        else:
            for han in phrase:
                py = self._single_pinyin(han, style, heteronym, errors, strict)
                pinyin_list.extend(py)

        return pinyin_list

    def convert_styles(self, pinyin_list, phrase, style, heteronym, errors,
                       strict, **kwargs):
        """转换多个汉字的拼音结果的风格"""
        for idx, item in enumerate(pinyin_list):
            han = phrase[idx]
            if heteronym:
                pinyin_list[idx] = [
                        self.convert_style(
                            han, orig_pinyin=x, style=style, strict=strict)
                        for x in item
                    ]
            else:
                orig_pinyin = item[0]
                pinyin_list[idx] = [
                    self.convert_style(
                        han, orig_pinyin=orig_pinyin, style=style,
                        strict=strict)]

        return pinyin_list

    def _single_pinyin(self, han, style, heteronym, errors, strict):
        """单字拼音转换.

        :param han: 单个汉字
        :param errors: 指定如何处理没有拼音的字符，详情请参考
                       :py:func:`~pypinyin.pinyin`
        :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                       是否严格遵照《汉语拼音方案》来处理声母和韵母，
                       详见 :ref:`strict`
        :return: 返回拼音列表，多音字会有多个拼音项
        :rtype: list
        """
        num = ord(han)
        # 处理没有拼音的字符
        if num not in PINYIN_DICT:
            return self.handle_nopinyin(
                han, style=style, errors=errors,
                heteronym=heteronym, strict=strict)

        pys = PINYIN_DICT[num].split(',')  # 字的拼音列表
        return [pys]

    def _convert_style(self, han, pinyin, style, strict, default,
                       **kwargs):
        if not kwargs:
            kwargs = {}
        kwargs['han'] = han

        return convert_style(pinyin, style, strict, default=default, **kwargs)

    def _convert_nopinyin_chars(self, chars, style, heteronym, errors, strict):
        """转换没有拼音的字符。

        """
        if callable_check(errors):
            return errors(chars)

        if errors == 'default':
            return chars
        elif errors == 'ignore':
            return None
        elif errors == 'replace':
            if len(chars) > 1:
                return ''.join(text_type('%x' % ord(x)) for x in chars)
            else:
                return text_type('%x' % ord(chars))


class _v2UConverter(V2UMixin, DefaultConverter):
    pass


class _neutralToneWith5Converter(NeutralToneWith5Mixin, DefaultConverter):
    pass


class _toneSandhiConverter(ToneSandhiMixin, DefaultConverter):
    pass


class UltimateConverter(DefaultConverter):
    def __init__(self, v_to_u=False, neutral_tone_with_five=False,
                 tone_sandhi=False, **kwargs):
        super(UltimateConverter, self).__init__(**kwargs)
        self._v_to_u = v_to_u
        self._neutral_tone_with_five = neutral_tone_with_five
        self._tone_sandhi = tone_sandhi

    def post_convert_style(self, han, orig_pinyin, converted_pinyin,
                           style, strict, **kwargs):
        post_data = super(UltimateConverter, self).post_convert_style(
            han, orig_pinyin, converted_pinyin, style, strict, **kwargs)
        if post_data is not None:
            converted_pinyin = post_data

        if self._v_to_u:
            post_data = _v2UConverter().post_convert_style(
                han, orig_pinyin, converted_pinyin, style, strict, **kwargs)
            if post_data is not None:
                converted_pinyin = post_data

        if self._neutral_tone_with_five:
            post_data = _neutralToneWith5Converter().post_convert_style(
                han, orig_pinyin, converted_pinyin, style, strict, **kwargs)
            if post_data is not None:
                converted_pinyin = post_data

        return converted_pinyin

    def post_pinyin(self, han, heteronym, pinyin, **kwargs):
        post_data = super(UltimateConverter, self).post_pinyin(
            han, heteronym, pinyin, **kwargs)
        if post_data is not None:
            pinyin = post_data

        if self._tone_sandhi:
            post_data = _toneSandhiConverter().post_pinyin(
                han, heteronym, pinyin, **kwargs)
            if post_data is not None:
                pinyin = post_data

        return pinyin


_mixConverter = UltimateConverter



from typing import Any, Dict, Tuple, Text

# 注音转换表
BOPOMOFO_REPLACE = ...  # type: Tuple[Tuple[Any]]
BOPOMOFO_TABLE = ...  # type: Dict[Text, Text]


class BopomofoConverter(object):
    def to_bopomofo(self, pinyin: Text, **kwargs: Any) -> Text: ...

    def to_bopomofo_first(self, pinyin: Text, **kwargs: Any) -> Text: ...

    def _pre_convert(self, pinyin: Text) -> Text: ...

converter = ...  # type: BopomofoConverter



# -*- coding: utf-8 -*-
"""Style.INITIALS 风格"""
from __future__ import unicode_literals

from pypinyin.constants import Style
from pypinyin.style import register
from pypinyin.style._utils import get_initials


@register(Style.INITIALS)
def convert(pinyin, **kwargs):
    strict = kwargs.get('strict', True)
    return get_initials(pinyin, strict)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from codecs import open
import os
import re
import sys

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

current_dir = os.path.dirname(os.path.realpath(__file__))

packages = [
    'pypinyin',
    'pypinyin.contrib',
    'pypinyin.seg',
    'pypinyin.style',
]

requirements = []
if sys.version_info[:2] < (2, 7):
    requirements.append('argparse')
if sys.version_info[:2] < (3, 4):
    requirements.append('enum34')
if sys.version_info[:2] < (3, 5):
    requirements.append('typing')
extras_require = {
    ':python_version<"2.7"': ['argparse'],
    ':python_version<"3.4"': ['enum34'],
    ':python_version<"3.5"': ['typing'],
}


def get_meta():
    meta_re = re.compile(r"(?P<name>__\w+__) = '(?P<value>[^']+)'")
    meta_d = {}
    with open(os.path.join(current_dir, 'pypinyin/__init__.py'),
              encoding='utf8') as fp:
        for match in meta_re.finditer(fp.read()):
            meta_d[match.group('name')] = match.group('value')
    return meta_d


def long_description():
    with open(os.path.join(current_dir, 'README.rst'),
              encoding='utf8') as fp:
        return fp.read()


meta_d = get_meta()
setup(
    name='pypinyin',
    version=meta_d['__version__'],
    description='汉字拼音转换模块/工具.',
    long_description=long_description(),
    long_description_content_type='text/x-rst',
    url='https://github.com/mozillazg/python-pinyin',
    author=meta_d['__author__'],
    author_email='mozillazg101@gmail.com',
    license=meta_d['__license__'],
    project_urls={
        'Documentation': 'https://pypinyin.readthedocs.io/',
        'Source': 'https://github.com/mozillazg/python-pinyin',
        'Tracker': 'https://github.com/mozillazg/python-pinyin/issues',
    },
    packages=packages,
    package_dir={'pypinyin': 'pypinyin'},
    include_package_data=True,
    install_requires=requirements,
    extras_require=extras_require,
    python_requires='>=2.6, !=3.0.*, !=3.1.*, !=3.2.*, <4',
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'pypinyin = pypinyin.__main__:main',
        ],
    },
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Utilities',
        'Topic :: Text Processing',
    ],
    keywords='pinyin, 拼音',
)



# -*- coding: utf-8 -*-
from functools import wraps

# 存储各拼音风格对应的实现
_registry = {}


def convert(pinyin, style, strict, default=None, **kwargs):
    """根据拼音风格把原始拼音转换为不同的格式

    :param pinyin: 原始有声调的单个拼音
    :type pinyin: unicode
    :param style: 拼音风格
    :param strict: 只获取声母或只获取韵母相关拼音风格的返回结果
                   是否严格遵照《汉语拼音方案》来处理声母和韵母，
                   详见 :ref:`strict`
    :type strict: bool
    :param default: 拼音风格对应的实现不存在时返回的默认值
    :param kwargs: 兼容后续可能会新增的关键字参数。当前包含如下关键字参数:
                   ``han``: 当前拼音对应的原始汉字。

    :return: 按照拼音风格进行处理过后的拼音字符串
    :rtype: unicode
    """
    if style in _registry:
        return _registry[style](pinyin, strict=strict, **kwargs)
    return default


def register(style, func=None):
    """注册一个拼音风格实现。
    自定义的函数应当使用 ``**kwargs`` 来兼容后续可能会新增的关键字参数，
    当前默认会传递如下参数：

    * ``pinyin``: 原始有声调的单个拼音
    * ``strict``: 是否开启 strict 模式
    * ``han``: 当前拼音对应的原始汉字

    ::

        @register('echo')
        def echo(pinyin, **kwargs):
            return pinyin

        # or
        register('echo', echo)
    """
    if func is not None:
        _registry[style] = func
        return

    def decorator(func):
        _registry[style] = func

        @wraps(func)
        def wrapper(pinyin, **kwargs):
            return func(pinyin, **kwargs)

        return wrapper
    return decorator


def auto_discover():
    """自动注册内置的拼音风格实现"""
    from pypinyin.style import (  # noqa
        initials,
        tone,
        finals,
        bopomofo,
        cyrillic,
        wadegiles,
        others,
    )



from typing import Any, Text


class FinalsConverter(object):
    def to_finals(self, pinyin: Text, **kwargs: Any) -> Text: ...

    def to_finals_tone(self, pinyin: Text, **kwargs: Any) -> Text: ...

    def to_finals_tone2(self, pinyin: Text, **kwargs: Any) -> Text: ...

    def to_finals_tone3(self, pinyin: Text, **kwargs: Any) -> Text: ...


converter = ...  # type: FinalsConverter



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import re

from pypinyin import phonetic_symbol
from pypinyin.style._tone_rule import right_mark_index
from pypinyin.style._constants import RE_TONE3, RE_TONE2
from pypinyin.style.tone import converter
from pypinyin.style._utils import (
    get_initials, replace_symbol_to_no_symbol,
    get_finals, replace_symbol_to_number
)

_re_number = re.compile(r'\d')


def to_normal(pinyin, v_to_u=False):
    """将 :py:attr:`~pypinyin.Style.TONE`、
    :py:attr:`~pypinyin.Style.TONE2` 或
    :py:attr:`~pypinyin.Style.TONE3` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音

    :param pinyin: :py:attr:`~pypinyin.Style.TONE`、
                   :py:attr:`~pypinyin.Style.TONE2` 或
                   :py:attr:`~pypinyin.Style.TONE3` 风格的拼音
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :return: :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import to_normal
      >>> to_normal('zhōng')
      'zhong'
      >>> to_normal('zho1ng')
      'zhong'
      >>> to_normal('zhong1')
      'zhong'
      >>> to_normal('lüè')
      'lve'
      >>> to_normal('lüè', v_to_u=True)
      'lüe'
    """
    s = tone_to_tone2(pinyin, v_to_u=True)
    s = tone2_to_normal(s)
    return _fix_v_u(pinyin, s, v_to_u=v_to_u)


def to_tone(pinyin):
    """将 :py:attr:`~pypinyin.Style.TONE2` 或
    :py:attr:`~pypinyin.Style.TONE3` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.TONE` 风格的拼音

    :param pinyin: :py:attr:`~pypinyin.Style.TONE2` 或
                   :py:attr:`~pypinyin.Style.TONE3` 风格的拼音
    :return: :py:attr:`~pypinyin.Style.TONE` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import to_tone
      >>> to_tone('zho1ng')
      'zhōng'
      >>> to_tone('zhong1')
      'zhōng'
    """
    if not _re_number.search(pinyin):
        return pinyin

    s = tone_to_tone2(pinyin)
    s = tone2_to_tone(s)
    return s


def to_tone2(pinyin, v_to_u=False, neutral_tone_with_five=False, **kwargs):
    """将 :py:attr:`~pypinyin.Style.TONE` 或
    :py:attr:`~pypinyin.Style.TONE3` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.TONE2` 风格的拼音

    :param pinyin: :py:attr:`~pypinyin.Style.TONE` 或
                   :py:attr:`~pypinyin.Style.TONE3` 风格的拼音
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :param neutral_tone_with_five: 是否使用 ``5`` 标识轻声
    :param kwargs: 用于兼容老版本的 ``neutral_tone_with_5`` 参数，当传入
                   ``neutral_tone_with_5`` 参数时，
                   将覆盖 ``neutral_tone_with_five`` 的值。
    :return: :py:attr:`~pypinyin.Style.TONE2` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import to_tone2
      >>> to_tone2('zhōng')
      'zho1ng'
      >>> to_tone2('zhong1')
      'zho1ng'
      >>> to_tone2('shang')
      'shang'
      >>> to_tone2('shang', neutral_tone_with_five=True)
      'sha5ng'
      >>> to_tone2('lüè')
      'lve4'
      >>> to_tone2('lüè', v_to_u=True)
      'lüe4'
    """
    if kwargs.get('neutral_tone_with_5', None) is not None:
        neutral_tone_with_five = kwargs['neutral_tone_with_5']
    s = tone_to_tone3(
        pinyin, v_to_u=True, neutral_tone_with_five=neutral_tone_with_five)
    s = tone3_to_tone2(s)
    return _fix_v_u(pinyin, s, v_to_u)


def to_tone3(pinyin, v_to_u=False, neutral_tone_with_five=False, **kwargs):
    """将 :py:attr:`~pypinyin.Style.TONE` 或
    :py:attr:`~pypinyin.Style.TONE2` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.TONE3` 风格的拼音

    :param pinyin: :py:attr:`~pypinyin.Style.TONE` 或
                   :py:attr:`~pypinyin.Style.TONE2` 风格的拼音
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :param neutral_tone_with_five: 是否使用 ``5`` 标识轻声
    :param kwargs: 用于兼容老版本的 ``neutral_tone_with_5`` 参数，当传入
                   ``neutral_tone_with_5`` 参数时，
                   将覆盖 ``neutral_tone_with_five`` 的值。
    :return: :py:attr:`~pypinyin.Style.TONE2` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import to_tone3
      >>> to_tone3('zhōng')
      'zhong1'
      >>> to_tone3('zho1ng')
      'zhong1'
      >>> to_tone3('shang')
      'shang'
      >>> to_tone3('shang', neutral_tone_with_five=True)
      'shang5'
      >>> to_tone3('lüè')
      'lve4'
      >>> to_tone3('lüè', v_to_u=True)
      'lüe4'
    """
    if kwargs.get('neutral_tone_with_5', None) is not None:
        neutral_tone_with_five = kwargs['neutral_tone_with_5']
    s = tone_to_tone2(
        pinyin, v_to_u=True, neutral_tone_with_five=neutral_tone_with_five)
    s = tone2_to_tone3(s)
    return _fix_v_u(pinyin, s, v_to_u)


def to_initials(pinyin, strict=True):
    """将 :py:attr:`~pypinyin.Style.TONE`、
    :py:attr:`~pypinyin.Style.TONE2` 、
    :py:attr:`~pypinyin.Style.TONE3` 或
    :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.INITIALS` 风格的拼音

    :param pinyin: :py:attr:`~pypinyin.Style.TONE`、
                   :py:attr:`~pypinyin.Style.TONE2` 、
                   :py:attr:`~pypinyin.Style.TONE3` 或
                   :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音
    :param strict: 返回结果是否严格遵照《汉语拼音方案》来处理声母和韵母，
                   详见 :ref:`strict`
    :return: :py:attr:`~pypinyin.Style.INITIALS` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import to_initials
      >>> to_initials('zhōng')
      'zh'

    """
    return get_initials(pinyin, strict=strict)


def to_finals(pinyin, strict=True, v_to_u=False):
    """将 :py:attr:`~pypinyin.Style.TONE`、
    :py:attr:`~pypinyin.Style.TONE2` 、
    :py:attr:`~pypinyin.Style.TONE3` 或
    :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.FINALS` 风格的拼音

    :param pinyin: :py:attr:`~pypinyin.Style.TONE`、
                   :py:attr:`~pypinyin.Style.TONE2` 、
                   :py:attr:`~pypinyin.Style.TONE3` 或
                   :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音
    :param strict: 返回结果是否严格遵照《汉语拼音方案》来处理声母和韵母，
                   详见 :ref:`strict`
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :return: :py:attr:`~pypinyin.Style.FINALS` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import to_finals
      >>> to_finals('zhōng')
      'ong'

    """
    new_pinyin = replace_symbol_to_no_symbol(pinyin).replace('v', 'ü')
    finals = get_finals(new_pinyin, strict=strict)
    finals = _fix_v_u(finals, finals, v_to_u)
    return finals


def to_finals_tone(pinyin, strict=True):
    """将 :py:attr:`~pypinyin.Style.TONE`、
    :py:attr:`~pypinyin.Style.TONE2` 或
    :py:attr:`~pypinyin.Style.TONE3` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.FINALS_TONE` 风格的拼音

    :param pinyin: :py:attr:`~pypinyin.Style.TONE`、
                   :py:attr:`~pypinyin.Style.TONE2` 或
                   :py:attr:`~pypinyin.Style.TONE3` 风格的拼音
    :param strict: 返回结果是否严格遵照《汉语拼音方案》来处理声母和韵母，
                   详见 :ref:`strict`
    :return: :py:attr:`~pypinyin.Style.FINALS_TONE` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import to_finals_tone
      >>> to_finals_tone('zhōng')
      'ōng'

    """
    finals = to_finals_tone2(pinyin, strict=strict)

    finals = tone2_to_tone(finals)

    return finals


def to_finals_tone2(pinyin, strict=True, v_to_u=False,
                    neutral_tone_with_five=False):
    """将 :py:attr:`~pypinyin.Style.TONE`、
    :py:attr:`~pypinyin.Style.TONE2` 或
    :py:attr:`~pypinyin.Style.TONE3` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.FINALS_TONE2` 风格的拼音

    :param pinyin: :py:attr:`~pypinyin.Style.TONE`、
                   :py:attr:`~pypinyin.Style.TONE2` 或
                   :py:attr:`~pypinyin.Style.TONE3` 风格的拼音
    :param strict: 返回结果是否严格遵照《汉语拼音方案》来处理声母和韵母，
                   详见 :ref:`strict`
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :param neutral_tone_with_five: 是否使用 ``5`` 标识轻声
    :return: :py:attr:`~pypinyin.Style.FINALS_TONE2` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import to_finals_tone2
      >>> to_finals_tone2('zhōng')
      'o1ng'

    """
    finals = to_finals_tone3(pinyin, strict=strict, v_to_u=v_to_u,
                             neutral_tone_with_five=neutral_tone_with_five)

    finals = tone3_to_tone2(finals, v_to_u=v_to_u)

    return finals


def to_finals_tone3(pinyin, strict=True, v_to_u=False,
                    neutral_tone_with_five=False):
    """将 :py:attr:`~pypinyin.Style.TONE`、
    :py:attr:`~pypinyin.Style.TONE2` 或
    :py:attr:`~pypinyin.Style.TONE3` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.FINALS_TONE3` 风格的拼音

    :param pinyin: :py:attr:`~pypinyin.Style.TONE`、
                   :py:attr:`~pypinyin.Style.TONE2` 或
                   :py:attr:`~pypinyin.Style.TONE3` 风格的拼音
    :param strict: 返回结果是否严格遵照《汉语拼音方案》来处理声母和韵母，
                   详见 :ref:`strict`
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :param neutral_tone_with_five: 是否使用 ``5`` 标识轻声
    :return: :py:attr:`~pypinyin.Style.FINALS_TONE3` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import to_finals_tone3
      >>> to_finals_tone3('zhōng')
      'ong1'

    """
    finals = to_finals(pinyin, strict=strict, v_to_u=v_to_u)
    if not finals:
        return finals

    numbers = _re_number.findall(replace_symbol_to_number(pinyin))
    if not numbers:
        if neutral_tone_with_five:
            numbers = ['5']
        else:
            return finals

    number = numbers[0]
    finals = finals + number

    return finals


def tone_to_normal(tone, v_to_u=False):
    """将 :py:attr:`~pypinyin.Style.TONE` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音

    :param tone: :py:attr:`~pypinyin.Style.TONE` 风格的拼音
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :return: :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import tone_to_normal
      >>> tone_to_normal('zhōng')
      'zhong'
      >>> tone_to_normal('lüè')
      'lve'
      >>> tone_to_normal('lüè', v_to_u=True)
      'lüe'
    """
    s = tone_to_tone2(tone, v_to_u=v_to_u)
    s = _re_number.sub('', s)
    return _v_to_u(s, v_to_u)


def tone_to_tone2(tone, v_to_u=False, neutral_tone_with_five=False, **kwargs):
    """将 :py:attr:`~pypinyin.Style.TONE` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.TONE2` 风格的拼音

    :param tone: :py:attr:`~pypinyin.Style.TONE` 风格的拼音
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :param neutral_tone_with_five: 是否使用 ``5`` 标识轻声
    :param kwargs: 用于兼容老版本的 ``neutral_tone_with_5`` 参数，当传入
                   ``neutral_tone_with_5`` 参数时，
                   将覆盖 ``neutral_tone_with_five`` 的值。
    :return: :py:attr:`~pypinyin.Style.TONE2` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import tone_to_tone2
      >>> tone_to_tone2('zhōng')
      'zho1ng'
      >>> tone_to_tone2('shang')
      'shang'
      >>> tone_to_tone2('shang', neutral_tone_with_5=True)
      'sha5ng'
      >>> tone_to_tone2('lüè')
      'lve4'
      >>> tone_to_tone2('lüè', v_to_u=True)
      'lüe4'
    """
    if kwargs.get('neutral_tone_with_5', None) is not None:
        neutral_tone_with_five = kwargs['neutral_tone_with_5']
    tone3 = tone_to_tone3(
        tone, v_to_u=v_to_u, neutral_tone_with_five=neutral_tone_with_five)
    s = tone3_to_tone2(tone3)
    return _v_to_u(s, v_to_u)


def tone_to_tone3(tone, v_to_u=False, neutral_tone_with_five=False, **kwargs):
    """将 :py:attr:`~pypinyin.Style.TONE` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.TONE3` 风格的拼音

    :param tone: :py:attr:`~pypinyin.Style.TONE` 风格的拼音
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :param neutral_tone_with_five: 是否使用 ``5`` 标识轻声
    :param kwargs: 用于兼容老版本的 ``neutral_tone_with_5`` 参数，当传入
                   ``neutral_tone_with_5`` 参数时，
                   将覆盖 ``neutral_tone_with_five`` 的值。
    :return: :py:attr:`~pypinyin.Style.TONE3` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import tone_to_tone3
      >>> tone_to_tone3('zhōng')
      'zhong1'
      >>> tone_to_tone3('shang')
      'shang'
      >>> tone_to_tone3('shang', neutral_tone_with_five=True)
      'shang5'
      >>> tone_to_tone3('lüè')
      'lve4'
      >>> tone_to_tone3('lüè', v_to_u=True)
      'lüe4'
    """
    if kwargs.get('neutral_tone_with_5', None) is not None:
        neutral_tone_with_five = kwargs['neutral_tone_with_5']
    tone3 = converter.to_tone3(tone)
    s = _improve_tone3(tone3, neutral_tone_with_five=neutral_tone_with_five)
    return _v_to_u(s, v_to_u)


def tone2_to_normal(tone2, v_to_u=False):
    """将 :py:attr:`~pypinyin.Style.TONE2` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音

    :param tone2: :py:attr:`~pypinyin.Style.TONE2` 风格的拼音
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :return: Style.NORMAL 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import tone2_to_normal
      >>> tone2_to_normal('zho1ng')
      'zhong'
      >>> tone2_to_normal('lüe4')
      'lve'
      >>> tone2_to_normal('lüe4', v_to_u=True)
      'lüe'
    """
    s = _re_number.sub('', tone2)
    s = _v_to_u(s, v_to_u)
    return _fix_v_u(tone2, s, v_to_u=v_to_u)


def tone2_to_tone(tone2):
    """将 :py:attr:`~pypinyin.Style.TONE2` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.TONE` 风格的拼音

    :param tone2: :py:attr:`~pypinyin.Style.TONE2` 风格的拼音
    :return: Style.TONE 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import tone2_to_tone
      >>> tone2_to_tone('zho1ng')
      'zhōng'
    """
    regex = re.compile(RE_TONE2.pattern.replace('$', ''))
    d = phonetic_symbol.phonetic_symbol_reverse
    string = tone2.replace('ü', 'v').replace('5', '').replace('0', '')

    def _replace(m):
        s = m.group(0)
        return d.get(s) or s

    return regex.sub(_replace, string).replace('v', 'ü')


def tone2_to_tone3(tone2, v_to_u=False):
    """将 :py:attr:`~pypinyin.Style.TONE2` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.TONE3` 风格的拼音

    :param tone2: :py:attr:`~pypinyin.Style.TONE2` 风格的拼音
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :return: :py:attr:`~pypinyin.Style.TONE3` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import tone2_to_tone3
      >>> tone2_to_tone3('zho1ng')
      'zhong1'
      >>> tone2_to_tone3('lüe4')
      'lve4'
      >>> tone2_to_tone3('lüe4', v_to_u=True)
      'lüe4'
    """
    tone3 = RE_TONE3.sub(r'\1\3\2', tone2)
    return _fix_v_u(tone2, tone3, v_to_u=v_to_u)


def tone3_to_normal(tone3, v_to_u=False):
    """将 :py:attr:`~pypinyin.Style.TONE3` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音

    :param tone3: :py:attr:`~pypinyin.Style.TONE3` 风格的拼音
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :return: :py:attr:`~pypinyin.Style.NORMAL` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import tone3_to_normal
      >>> tone3_to_normal('zhong1')
      'zhong'
      >>> tone3_to_normal('lüe4')
      'lve'
      >>> tone3_to_normal('lüe4', v_to_u=True)
      'lüe'
    """
    s = _re_number.sub('', tone3)
    s = _v_to_u(s, v_to_u)
    return _fix_v_u(tone3, s, v_to_u=v_to_u)


def tone3_to_tone(tone3):
    """将 :py:attr:`~pypinyin.Style.TONE3` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.TONE` 风格的拼音

    :param tone3: :py:attr:`~pypinyin.Style.TONE3` 风格的拼音
    :return: :py:attr:`~pypinyin.Style.TONE` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import tone3_to_tone
      >>> tone3_to_tone('zhong1')
      'zhōng'
    """
    tone2 = tone3_to_tone2(tone3, v_to_u=True)
    return tone2_to_tone(tone2)


def tone3_to_tone2(tone3, v_to_u=False):
    """将 :py:attr:`~pypinyin.Style.TONE3` 风格的拼音转换为
    :py:attr:`~pypinyin.Style.TONE2` 风格的拼音

    :param tone3: :py:attr:`~pypinyin.Style.TONE3` 风格的拼音
    :param v_to_u: 是否使用 ``ü`` 代替原来的 ``v``，
                   当为 False 时结果中将使用 ``v`` 表示 ``ü``
    :return: :py:attr:`~pypinyin.Style.TONE2` 风格的拼音

    Usage::

      >>> from pypinyin.contrib.tone_convert import tone3_to_tone2
      >>> tone3_to_tone2('zhong1')
      'zho1ng'
      >>> tone3_to_tone2('lüe4')
      'lve4'
      >>> tone3_to_tone2('lüe4', v_to_u=True)
      'lüe4'
    """
    no_number_tone3 = tone3_to_normal(tone3)
    mark_index = right_mark_index(no_number_tone3)
    if mark_index is None:
        mark_index = len(no_number_tone3) - 1
    before = no_number_tone3[:mark_index + 1]
    after = no_number_tone3[mark_index + 1:]

    number = _get_number_from_pinyin(tone3)
    if number is None:
        return tone3

    s = '{}{}{}'.format(before, number, after)
    return _fix_v_u(tone3, s, v_to_u=v_to_u)


def _improve_tone3(tone3, neutral_tone_with_five=False):
    number = _get_number_from_pinyin(tone3)
    if number is None and neutral_tone_with_five and tone3 != '':
        tone3 = '{}5'.format(tone3)
    return tone3


def _get_number_from_pinyin(pinyin):
    numbers = _re_number.findall(pinyin)
    if numbers:
        number = numbers[0]
    else:
        number = None
    return number


def _v_to_u(pinyin, replace=False):
    if not replace:
        return pinyin
    return pinyin.replace('v', 'ü')


def _fix_v_u(origin_py, new_py, v_to_u):
    if not v_to_u:
        return new_py.replace('ü', 'v')

    return _v_to_u(new_py, replace=True)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from pypinyin.contrib.tone_convert import tone_to_tone2, tone2_to_tone

_re_num = re.compile(r'\d')


class ToneSandhiMixin(object):
    """

    按普通话变调规则处理拼音：

    * https://en.wikipedia.org/wiki/Standard_Chinese_phonology#Tone_sandhi
    * https://studycli.org/zh-CN/learn-chinese/tone-changes-in-mandarin/

    """  # noqa

    def post_pinyin(self, han, heteronym, pinyin_list, **kwargs):
        ret = super(ToneSandhiMixin, self).post_pinyin(
            han, heteronym, pinyin_list, **kwargs)
        if ret is not None:
            pinyin_list = ret

        pinyin_list = self._third_tone(han, pinyin_list)
        pinyin_list = self._bu(han, pinyin_list)
        pinyin_list = self._yi(han, pinyin_list)

        return pinyin_list

    def _third_tone(self, han, pinyin_list):
        """

        Third tone sandhi:

        The principal rule of third tone sandhi is:

            When there are two consecutive third-tone syllables, the first of them is pronounced with second tone.

        For example, lǎoshǔ 老鼠 ("mouse") comes to be pronounced láoshǔ [lau̯˧˥ʂu˨˩]. It has been investigated whether the rising contour (˧˥) on the prior syllable is in fact identical to a normal second tone; it has been concluded that it is, at least in terms of auditory perception.[1]: 237 

        When there are three or more third tones in a row, the situation becomes more complicated, since a third tone that precedes a second tone resulting from third tone sandhi may or may not be subject to sandhi itself. The results may depend on word boundaries, stress, and dialectal variations. General rules for three-syllable third-tone combinations can be formulated as follows:

            If the first word is two syllables and the second word is one syllable, then the first two syllables become second tones. For example, bǎoguǎn hǎo 保管好 ("to take good care of") takes the pronunciation báoguán hǎo [pau̯˧˥kwan˧˥xau̯˨˩˦].
            If the first word is one syllable, and the second word is two syllables, the second syllable becomes second tone, but the first syllable remains third tone. For example: lǎo bǎoguǎn 老保管 ("to take care of all the time") takes the pronunciation lǎo báoguǎn [lau̯˨˩pau̯˧˥kwan˨˩˦].

        Some linguists have put forward more comprehensive systems of sandhi rules for multiple third tone sequences. For example, it is proposed[1]: 248  that modifications are applied cyclically, initially within rhythmic feet (trochees; see below), and that sandhi "need not apply between two cyclic branches".

        """  # noqa
        tone2_pinyin_list = [tone_to_tone2(x[0]) for x in pinyin_list]
        if '3' not in ''.join(tone2_pinyin_list):
            return pinyin_list

        changed = False
        third_num = 0
        for pinyin in tone2_pinyin_list:
            if '3' in pinyin:
                third_num += 1
            else:
                third_num = 0

        if third_num == 2:
            for i, v in enumerate(tone2_pinyin_list):
                if '3' in v:
                    tone2_pinyin_list[i] = v.replace('3', '2')
                    changed = True
                    break

        elif third_num > 2:
            n = 1
            for i, v in enumerate(tone2_pinyin_list):
                if '3' in v:
                    if n == third_num:
                        break
                    tone2_pinyin_list[i] = v.replace('3', '2')
                    changed = True
                    n += 1

        if changed:
            return [[tone2_to_tone(x)] for x in tone2_pinyin_list]

        return pinyin_list

    def _bu(self, han, pinyin_list):
        """

        For 不 bù:

            不 is pronounced with second tone when followed by a fourth tone syllable.

                Example: 不是 (bù+shì, "to not be") becomes búshì [pu˧˥ʂɻ̩˥˩]

            In other cases, 不 is pronounced with fourth tone. However, when used between words in an A-not-A question, it may become neutral in tone (e.g., 是不是 shìbushì).

        """  # noqa
        if '不' not in han:
            return pinyin_list

        tone2_pinyin_list = [tone_to_tone2(x[0]) for x in pinyin_list]
        changed = False

        for i, h in enumerate(han):
            current_pinyin = tone2_pinyin_list[i]
            if h == '不' and i < len(han) - 1:
                next_pinyin = tone2_pinyin_list[i+1]
                if '4' in next_pinyin:
                    tone2_pinyin_list[i] = current_pinyin.replace('4', '2')
                    changed = True
                else:
                    tone2_pinyin_list[i] = _re_num.sub('4', current_pinyin)
                    changed = True
            elif h == '不':
                tone2_pinyin_list[i] = _re_num.sub('4', current_pinyin)
                changed = True

        if changed:
            return [[tone2_to_tone(x)] for x in tone2_pinyin_list]

        return pinyin_list

    def _yi(self, han, pinyin_list):
        """

        For 一 yī:

            一 is pronounced with second tone when followed by a fourth tone syllable.

                Example: 一定 (yī+dìng, "must") becomes yídìng [i˧˥tiŋ˥˩]

            Before a first, second or third tone syllable, 一 is pronounced with fourth tone.

                Examples：一天 (yī+tiān, "one day") becomes yìtiān [i˥˩tʰjɛn˥], 一年 (yī+nián, "one year") becomes yìnián [i˥˩njɛn˧˥], 一起 (yī+qǐ, "together") becomes yìqǐ [i˥˩t͡ɕʰi˨˩˦].

            When final, or when it comes at the end of a multi-syllable word (regardless of the first tone of the next word), 一 is pronounced with first tone. It also has first tone when used as an ordinal number (or part of one), and when it is immediately followed by any digit (including another 一; hence both syllables of the word 一一 yīyī and its compounds have first tone).
            When 一 is used between two reduplicated words, it may become neutral in tone (e.g. 看一看 kànyikàn ("to take a look of")).

        """  # noqa
        if '一' not in han:
            return pinyin_list

        tone2_pinyin_list = [tone_to_tone2(x[0]) for x in pinyin_list]
        changed = False

        for i, h in enumerate(han):
            current_pinyin = tone2_pinyin_list[i]
            if h == '一' and i < len(han) - 1:
                next_pinyin = tone2_pinyin_list[i + 1]
                if '4' in next_pinyin:
                    tone2_pinyin_list[i] = current_pinyin.replace('4', '2')
                    changed = True
                else:
                    tone2_pinyin_list[i] = _re_num.sub('4', current_pinyin)
                    changed = True
            elif h == '一':
                tone2_pinyin_list[i] = _re_num.sub('1', current_pinyin)
                changed = True

        if changed:
            return [[tone2_to_tone(x)] for x in tone2_pinyin_list]

        return pinyin_list
